' -----------------------------------------------------------------------------
' This file is an example of automated Multibeam processing using COM scripting
' from Windows' inbuilt Visual Basic scripting language
' Written by David Millington and Briony Hutton, Myriax Pty Ltd, May 2008
' Updated by Suenor Woon, Echoview Software, Arpil 2017

' -----------------------------------------------------------------------------
' Strict syntax checking
Option Explicit

' -----------------------------------------------------------------------------
' Run Echoview, if it's not already running, or connect to Echoview, if it is
' already running
Dim EvApp: Set EvApp = CreateObject("EchoviewCom.EvApplication")

' This line calls the main working function. 
ProcessAllDataFilesInFolder()

' -----------------------------------------------------------------------------
' This method is the main working method - it loops through all files in the
' data folder, and if they are data files with a .ddf extension it passes them
' on to ProcessDataFile(...) to do the actual work
Function ProcessAllDataFilesInFolder()
  ' Create a file system object to help handle files
  Dim FSO: Set FSO = CreateObject("Scripting.FileSystemObject")

  ' Make sure the data folder exists
  If Not FSO.FolderExists("C:\Echoview Software\Tutorials\IntroToMultibeamTargetDataProcessing") Then
     MsgBox "Error: The data file path does not exist"
     WScript.Quit ' If there are no input files, cannot continue
  End If

  ' Create a folder object to help handle using the data file input folder,
  ' and loop through all files in it
  Dim DataFileFolder: Set DataFileFolder = FSO.GetFolder("C:\Echoview Software\Tutorials\IntroToMultibeamTargetDataProcessing")
  If DataFileFolder.Files.Count <> 0 Then
    ' Keep a count of how many files have been processed
    Dim NumFilesProcessed
    NumFilesProcessed = 0

    ' The actual loop
    Dim File
    For Each File In DataFileFolder.Files
      ' If this is a data file of the type we are looking for
      If FSO.GetExtensionName(File.Name) = "ddf" Then
        ProcessDataFile(File.Name) ' This does the actual work for this .ddf file
        NumFilesProcessed = NumFilesProcessed + 1
      End If
    Next

    ' Finished looping through all files, so print a message saying how many
    ' were processed
    MsgBox "Done! " & NumFilesProcessed & " file(s) were processed."
  Else
     ' This occurs if DataFileFolder.Files.Count (the number of files in the
     ' folder) equalled 0
     MsgBox "Error: The data file path contains no data files of the type specified"
  End If
End Function

' -----------------------------------------------------------------------------
' This function creates an EV file from a template, adds a .ddf file, exports single target data from a
' virtual variable, imports that exported file into another fileset, then saves the EV file.
Function ProcessDataFile(strDataFileName)
  ' Create a file system object to help handle files
  Dim FSO: Set FSO = CreateObject("Scripting.FileSystemObject")

  ' Check the template exists
  If Not FSO.FileExists("C:\Echoview Software\Tutorials\IntroToMultibeamTargetDataProcessing\Templates\TutorialTemplate.EV") Then
     MsgBox "Error: the template EV file does not exist."
     WScript.Quit ' if the template doesn't exist, cannot continue
  End If

  ' Create a new EV file and import the template
  Dim EvFile: Set EvFile = EvApp.NewFile("C:\Echoview Software\Tutorials\IntroToMultibeamTargetDataProcessing\Templates\TutorialTemplate.EV")
  If EvFile Is Nothing Then
     MsgBox "There was an error creating a new EV file using the template file."
  End If

  ' Add the data file to the first fileset .
  If Not EvFile.Filesets(0).DataFiles.Add("C:\Echoview Software\Tutorials\IntroToMultibeamTargetDataProcessing\" & strDataFileName) Then
     MsgBox "There was an error adding the data file."
  End If

  ' Find the single targets variable in this EV file and export it
  Dim EvSTVar: Set EvSTVar = EvFile.Variables.FindByName("Single targets >20cm target length")
  Dim strSTExportFileName
  strSTExportFileName = "Script " & FSO.GetBaseName(strDataFileName) & " single targets GT 20cm target length.target.csv"
  If Not EvSTVar.ExportData("C:\Echoview Software\Tutorials\IntroToMultibeamTargetDataProcessing" & strSTExportFileName, -1, -1) Then ' -1 for first and last ping means 'export all'
     MsgBox "There was an error exporting the single targets variable to the file: "  & strSTExportFileName
  End If

  ' Add the exported single target file to the "Export single targets" fileset
  EvFile.Filesets.FindByName("Exported single targets").DataFiles.Add("C:\Echoview Software\Tutorials\IntroToMultibeamTargetDataProcessing\" & strSTExportFileName)

  ' Detect fish tracks on the original single target echogram
  EvSTVar.DetectFishTracks "Greater than 20cm target length", -1, -1

  ' Export an analysis by cells - Fish tracks
  EvSTVar.ExportFishTracksByRegions "C:\Echoview Software\Tutorials\IntroToMultibeamTargetDataProcessing\" & "Scripted " & FSO.GetBaseName(strDataFileName) & " fish tracks GT 20cm target length.csv", EvFile.RegionClasses.FindByName("Greater than 20cm target length")

  ' Save the EV file
  Dim strEvFileName
  strEvFileName = FSO.GetBaseName(strDataFileName) & " Greater than 20cm target length.EV"
  If Not EvFile.SaveAs("C:\Echoview Software\Tutorials\IntroToMultibeamTargetDataProcessing\" & strEvFileName) Then
     MsgBox "There was an error saving the EV file: " & strEvFileName

  End If
  'Close EV File and pass back to main program, so that a new EF file is created from a template and the script works through another data file.
  EvFile.Close
End Function
